import _extends from "@babel/runtime/helpers/esm/extends";
import React, { useContext } from 'react';
import InitialPropsContext from './initialPropsContext';
import withInitialProps from './withInitialProps';
import createClientContextFactoryParams from './createClientContextFactoryParams';

var EMPTY_GET_INITIAL_PROPS = function EMPTY_GET_INITIAL_PROPS() {
  return Promise.resolve({});
};

var createUniversalPage = function createUniversalPage(contextFactory, routeInfo) {
  return function (_ref) {
    var Page = _ref["default"];
    // check if the Page page has a getInitialProps, if not put a resolve with an empty object
    Page.getInitialProps = typeof Page.getInitialProps === 'function' ? Page.getInitialProps : EMPTY_GET_INITIAL_PROPS; // check if we're on the client

    if (typeof window !== 'undefined') {
      // check if we have already data for this component on the window
      if (typeof window.__INITIAL_PROPS__ !== 'undefined') {
        // make a copy of the content safely
        var initialProps = _extends({}, window.__INITIAL_PROPS__); // remove the variable of the window before returning the component


        window.__INITIAL_PROPS__ = undefined; // resolve the promise with the initialProps passed to the client

        return Promise.resolve(function (props) {
          return /*#__PURE__*/React.createElement(Page, Object.assign({}, initialProps, props));
        });
      } // create the context to be used as param on the getInitialProps
      // TODO: Maybe this is no needed as we already have created it?


      return contextFactory(createClientContextFactoryParams()) // now, we have to create the Page to be rendered on the client with all the info
      .then(function (context) {
        return withInitialProps({
          context: context,
          routeInfo: routeInfo
        })(Page);
      });
    } // we're in the server: Create a component that gets the initialProps from context
    // this context has been created on the `ssrWithComponentWithInitialProps`


    var ServerPage = function ServerPage(props) {
      var _useContext = useContext(InitialPropsContext),
          initialProps = _useContext.initialProps;

      return /*#__PURE__*/React.createElement(Page, Object.assign({}, props, initialProps));
    }; // recover the displayName from the original page


    ServerPage.displayName = Page.displayName; // detect if the page has getInitialProps and wrap it with the routeInfo
    // if we don't have any getInitialProps, just use a empty function returning an empty object

    ServerPage.getInitialProps = function (context) {
      return Page.getInitialProps({
        context: context,
        routeInfo: routeInfo
      });
    }; // return the component to be used on the server


    return ServerPage;
  };
};

export default (function (contextFactory, importPage) {
  return function (routeInfo, done) {
    importPage().then(createUniversalPage(contextFactory, routeInfo)).then(function (Page) {
      done(null, Page);
    });
  };
});